%Edge Detection

addpath ./lib/qtfm/

file_name='./../Data/ani1_mig.HH';
D1=load_history(file_name);
fidd1=fopen(D1.in,'r','ieee-be');
arr_size=[D1.n(1),D1.n(2)];
nelem=arr_size(1)*arr_size(2);
f(1:nelem) = fread(fidd1,nelem,'float32');
image1=reshape(f,[D1.n(1),D1.n(2)]);


sigma1=5;
sigma2=5;
wa=.05;
wb=.05;
e=.5;

image1=transpose(image1);

t1=zeros(size(image1,1)+1,size(image1,2)+1);
t2=zeros(size(image1,1)+1,size(image1,2)+1);
n1=size(image1,1);
n2=size(image1,2);

eps=1;
for i=1:n1
  for j=1:n2
    t1(i,j)=image1(i,j);
    t2(i+1,j+1)=image1(i,j);
  end
end

image1=t1;
image2=t2;



eps1=e;
n1=size(image1,1);
n2=size(image1,2);
u=-n2/2:n2/2-1;
v=-n1/2:n1/2-1;


[U,V]=meshgrid(u,v);

z1=zeros(size(image1,1),size(image1,2));
z2=zeros(size(image1,1),size(image1,2));



qi1=quaternion(image1,z1,z1,z1);
qi2=quaternion(image2,z2,z2,z2);
qi1ft=qfft2(qi1);
qi2ft=qfft2(qi2);

fti1=fft2(image1);
fti2=fft2(image2);




%Construct Quaternion Gabor Filer
%gausian
g=exp(-(U.^2)./(2*sigma1)).*exp(-((eps.*V).^2)./(2*sigma2^2));

%finding u0 and v0
f1=fft(image1,[],1);
u0=max(max(real(f1)));
f2=fft(image2,[],2);
v0=max(max(real(f2)));
%quaternion component
w1=wa*U;
w2=wb*V;

%Components of QGF and ffts of components
hee=g.*(cos(w1).*cos(w2));
fthee=fft2(hee);
hoe=g.*(sin(w1).*cos(w2));
fthoe=fft2(hoe);
heo=g.*(cos(w1).*sin(w2));
ftheo=fft2(heo);
hoo=g.*(sin(w1).*sin(w2));
fthoo=fft2(hoo);

%convolve filter with image a
ftkee1=fthee.*fti1;
kee1=ifft2(ftkee1);
ftkoe1=fthoe.*fti1;
koe1=ifft2(ftkoe1);
ftkeo1=ftheo.*fti1;
keo1=ifft2(ftkeo1);
ftkoo1=fthoo.*fti1;
koo1=ifft2(ftkoo1);

%convolve filter with image b
ftkee2=fthee.*fti2;
kee2=ifft2(ftkee2);
ftkoe2=fthoe.*fti2;
koe2=ifft2(ftkoe2);
ftkeo2=ftheo.*fti2;
keo2=ifft2(ftkeo2);
ftkoo2=fthoo.*fti2;
koo2=ifft2(ftkoo2);

%necessary functions for image1
nphi1=-2*(keo1.*koo1+kee1.*koe1);
dphi1=kee1.^2-koe1.^2+keo1.^2-koo1.^2;
ntheta1=-2*(koe1.*koo1+kee1.*keo1);
dtheta1=kee1.^2+koe1.^2-keo1.^2-koo1.^2;

%necessary functions for image2
nphi2=-2*(keo2.*koo2+kee2.*koe2);
dphi2=kee2.^2-koe2.^2+keo2.^2-koo2.^2;
ntheta2=-2*(koe2.*koo2+kee2.*keo2);
dtheta2=kee2.^2+koe2.^2-keo2.^2-koo2.^2;

%phase components for image1
phi1=atan2(nphi1,dphi1)/2;
theta1=atan2(ntheta1,dtheta1)/2;


%phase components for image2
phi2=atan2(nphi2,dphi2)/2;
theta2=atan2(ntheta2,dtheta2)/2;


%derivatives of phase components

dnphi1=zeros(size(image1));
ddphi1=zeros(size(image1));
dntheta1=zeros(size(image1));
ddtheta1=zeros(size(image1));

dnphi2=zeros(size(image2));
ddphi2=zeros(size(image2));
dntheta2=zeros(size(image2));
ddtheta2=zeros(size(image2));

dnphi1(:,2:n2-1)=(nphi1(:,3:n2)-nphi1(:,1:n2-2))/2;
ddphi1(:,2:n2-1)=(dphi1(:,3:n2)-dphi1(:,1:n2-2))/2;
ddtheta1(2:n1-1,:)=(dtheta1(3:n1,:)-dtheta1(1:n1-2,:))/2;
dntheta1(2:n1-1,:)=(ntheta1(3:n1,:)-ntheta1(1:n1-2,:))/2;

dnphi2(:,2:n2-1)=(nphi2(:,3:n2)-nphi2(:,1:n2-2))/2;
ddphi2(:,2:n2-1)=(dphi2(:,3:n2)-dphi2(:,1:n2-2))/2;
ddtheta2(2:n1-1,:)=(dtheta2(3:n1,:)-dtheta2(1:n1-2,:))/2;
dntheta2(2:n1-1,:)=(ntheta2(3:n1,:)-ntheta2(1:n1-2,:))/2;

dxphi1=0*dnphi1;
dytheta1=0*dntheta1;

dxphi2=0*dnphi2;
dytheta2=0*dntheta2;

ha=find(nphi1.^2+dphi1.^2 > eps1);
hb=find(ntheta1.^2+dtheta1.^2 > eps1);

dxphi1(ha)=((dphi1(ha).*dnphi1(ha))-(nphi1(ha).*ddphi1(ha)))./(nphi1(ha).^2+ ...
                                                  dphi1(ha).^2);

dytheta1(hb)=((dtheta1(hb).*dntheta1(hb))-(ntheta1(hb).*ddtheta1(hb)))./(ntheta1(hb).^2+dtheta1(hb).^2);
  
ha=find(nphi2.^2+dphi2.^2 > eps1);
hb=find(ntheta2.^2+dtheta2.^2 > eps1);

dxphi2(ha)=((dphi2(ha).*dnphi2(ha))-(nphi2(ha).*ddphi2(ha)))./(nphi2(ha).^2+ ...
                                                  dphi2(ha).^2);

dytheta2(hb)=((dtheta2(hb).*dntheta2(hb))-(ntheta2(hb).*ddtheta2(hb)))./(ntheta2(hb).^2+dtheta2(hb).^2);


%disparities(forward)
dx=0*image1;
dy=0*image2;
h1=find(abs(dxphi1) > eps1);
h2=find(abs(dytheta1) > eps1);
dx(h1)=(phi2(h1)-phi1(h1))./(dxphi1(h1));
dy(h2)=(theta2(h2)-theta1(h2))./(dytheta1(h2));



dxs=fftshift(dx);
dys=fftshift(dy);

%reverse
dx2=0*image1;
dy2=0*image2;
h1=find(abs(dxphi2) > eps1);
h2=find(abs(dytheta2) > eps1);
dx2(h1)=(phi1(h1)-phi2(h1))./(dxphi2(h1));
dy2(h2)=(theta1(h2)-theta2(h2))./(dytheta2(h2));

dx2s=fftshift(dx2);
dy2s=fftshift(dy2);

%pseudo-average
dxn=(dxs-dx2s)/2;
dyn=(dys-dy2s)/2;


mag=sqrt(dxn.^2+dyn.^2);
magf=sqrt(dxs.^2+dys.^2);
magb=sqrt(dx2s.^2+dy2s.^2);

figure;
subplot(1,2,1)
imagesc(image1);
z=[min(min(image1)) max(max(image1))];
caxis(.1*z);
colormap gray;

subplot(1,2,2);
imagesc(magf);
colormap(1-gray)

