function Y = qdft2(X, A, L)
% QDFT2 Discrete quaternion 2D Fourier transform.
%
% This function implements the discrete quaternion 2D Fourier transform of X.
% It does so using direct evaluation using a matrix product, and it is
% intended mainly for comparing results against the fast implementation
% QFFT2.

% Copyright  2005 Stephen J. Sangwine and Nicolas Le Bihan.
% See the file : Copyright.m for further details.

error(nargchk(1, 3, nargin)), error(nargoutchk(0, 1, nargout))

if nargin == 1
    A = quaternion(1, 1, 1); % No values were given for A or L,
    L = 'L';                 % so supply the default values.
end

if nargin == 2 % One of A or L was specified, but we don't know which.
    if A == 'L' | A == 'R'
        L = A;                 % A has a sensible value for L, so copy it
        A = quaternion(1,1,1); % into L and supply a default value for A.
    else
        L = 'L'; % Assume that A is an axis, and supply a default value for L.
    end
end

if size(A) ~= [1, 1]
    error('The transform axis cannot be a matrix or vector.');
end

if ~isa(A, 'quaternion') | ~ispure(A)
    error('The transform axis must be a pure quaternion.')
end

if L ~= 'L' & L ~= 'R'
    error('L must have the value ''L'' or ''R''.');
end

A = unit(A); % Ensure that A is a unit (pure) quaternion.

% Compute the transform. This is done by row/column separation, that is we
% compute the QDFT of the rows, then the QDFT of the columns. This is faster
% than a direct implementation, and easier, because the direct implementation
% would require a block matrix for the exponentials, which Matlab cannot
% support.

Y = qdft(qdft(X, A, L).', A, L).';
