function y = angle(x)
% ANGLE or phase of quaternion.
% (Quaternion overloading of standard Matlab function.)
%
% If Q = A + mu .* B where A and B are real/complex, and mu is a unit pure
% quaternion, then angle(Q) = B. For pure quaternions, angle(Q) = pi/2.

% Copyright  2005 Stephen J. Sangwine and Nicolas Le Bihan.
% See the file : Copyright.m for further details.

error(nargchk(1, 1, nargin)), error(nargoutchk(0, 1, nargout))

if ispure(x)
    
    % x is a pure quaternion, with no scalar part, and by definition, the
    % angle is a right-angle. Return a matrix of the same size as x, with
    % pi/2 at every position.
    
    y = ones(size(x)) .* pi ./ 2;
    
else
    
    % x is a full quaternion, and we handle this by constructing an equivalent
    % complex value, so that we can use the standard Matlab angle function. We
    % take the absolute value of the vector part - this means the imaginary
    % part of the complex number is always positive and therefore the angle
    % returned is between 0 and pi inclusive. This is normal for quaternions,
    % since the vector part is a direction in 3-space. The opposite direction
    % is represented by reversing the vector part, not by adding pi to the angle.
    
    y = angle(complex(s(x), abs(v(x))));
    
end
