function Y = exp(X)
% EXP  Exponential
% (Quaternion overloading of standard Matlab function.)

% Copyright  2005 Stephen J. Sangwine and Nicolas Le Bihan.
% See the file : Copyright.m for further details.

error(nargchk(1, 1, nargin)), error(nargoutchk(0, 1, nargout)) 

if ispure(X)
    
    % X is a pure quaternion (array).
    
    theta = abs(X); % NB: theta could be complex, if X is a complex quaternion.
    
    undefined = abs(theta) < eps;
    
    if any(undefined)
        
        % We have a problem -- one or more of the quaternions has no defined axis
        % and in these cases the exponential must be zero. The cosine and sine of
        % theta will take care of this, but we have to prevent a NaN being
        % computed for the undefined axis/axes. We do this by setting the axis to
        % an arbitrary value (qi) for all elements which have an undefined axis.
        
        mu = axis(X + undefined .* qi);
    else
        mu = axis(X); % There will be no problem in this case.
    end
    
    Y = cos(theta) + mu .* sin(theta); % If theta is complex, the cosine and sine
                                       % will yield cosh and I.*sinh of theta.
else
    
    % X is a full quaternion (matrix).   We use a recursive call to
    % compute the exponential of the vector part,  and the standard
    % Matlab function to compute the exponential of the scalar part.
    % The result is the product of the two.
    

    Y = exp(s(X)) .* exp(v(X));
end
